#include "tester_main.h"


TESTER_ProjectTypeDef TESTER = {0};


/**s
  * @brief    Инициализация всех модулей для работы тестера
  */
void TESTER_Init(TESTER_ProjectTypeDef *tester)
{
  TESTER_InterfaceInit(&tester->SwStart, &tester->leds);
  tester->leds.LED1.period = 100;
  
  tester->delay = 250;
  tester->delay_en = 0;
  
  tester->func.disable_reset_call = 0;
  tester->htest = &hTestDiode;
  TESTER_HandleInit(tester->htest, &tester->leds);
  
  tester->hmodbus = &hmodbus1;
  tester->mbdata = &MB_DATA;
  MODBUS_FirstInit();
}


/**s
  * @brief    Функция подготовки тестера к while(1)
  */
void TESTER_pre_while(TESTER_ProjectTypeDef *tester)
{
  TESTER_Set_Default_Settings(tester);
  TESTER_UpdateSettings(tester->htest, tester->mbdata);
  
  RS_Receive_IT(tester->hmodbus, &MODBUS_MSG);
  tester->leds.LED1.period = LED_BLINK_AS_ON;
  tester->f.flag_init_done = 1;
}



/**
  * @brief    Главная функция работы тестера
  */
void TESTER_main_while(TESTER_ProjectTypeDef *tester)
{
  if(tester->delay_en)
    msDelay(tester->delay);
  
  /* ТЕСТ В ОБРАТНОМ ВКЛЮЧЕНИИ */
  if(tester->func.test_diode_neg)
  {
    tester->f.flag_test_active = 1;
    TESTER_TestDiode_NegativePower(tester->htest);
    
    /* Запись данных в modbus */
    tester->mbdata->InRegs.PeakVoltage = tester->htest->DiodePeakVolt*1000;
    
    if(tester->func.disable_reset_call == 0)
      tester->func.test_diode_neg = 0;
    
    tester->f.flag_test_active = 0;
  }
    
  /* ТЕСТ В ПРЯМОМ ВКЛЮЧЕНИИ */
  if(tester->func.test_diode_pos)
  {
    tester->f.flag_test_active = 1;
    TESTER_TestDiode_PositivePower(tester->htest);
    
    /* Запись данных в modbus */
    tester->mbdata->InRegs.ForwardVoltage = tester->htest->DiodeForwardVolt*1000;
    
    if(tester->func.disable_reset_call == 0)
      tester->func.test_diode_pos = 0;
    
    tester->f.flag_test_active = 0;
  }
  /* ТЕСТ ПЕРЕХОДА ИЗ ПРЯМОГО В ОБРАТНОЕ ВКЛЮЧЕНИЯ*/
  if(tester->func.test_diode_posneg)
  {
    tester->f.flag_test_active = 1;
    TESTER_TestDiode_PosNegPower(tester->htest);
    
    /* Запись данных в modbus */
    tester->mbdata->InRegs.PeakVoltage = tester->htest->DiodePeakVolt*1000;
    tester->mbdata->InRegs.ForwardVoltage = tester->htest->DiodeForwardVolt*1000;
    
    if(tester->func.disable_reset_call == 0)
      tester->func.test_diode_posneg = 0;
    
    tester->f.flag_test_active = 0;
  }
}



/**
  * @brief    Функция работы интерфейса
  */
void TESTER_InterfaceHandle(TESTER_ProjectTypeDef *tester)
{
  if(tester->f.flag_init_done == 0)
    return;
  
  
  
  
  /* считывание режима тестера с модбас */
  tester->mode = *((TESTER_TestModeTypeDef *)&tester->mbdata->Coils);
  
  /* если кнопка нажата или пришла соответствующая комманда модбас */
  if(TESTER_ReadSwichStart(&tester->SwStart) || tester->mbdata->Coils.StartTest)
  {
    /* Обновление настроек тестера */
    TESTER_UpdateSettings(tester->htest, tester->mbdata);
    switch(tester->mode)
    {
      case TEST_POSITIVE:
        tester->func.test_diode_pos = 1;
        break;
          
      case TEST_NEGATIVE:
        tester->func.test_diode_neg = 1;
        break;
          
      case TEST_POSNEG:
        tester->func.test_diode_posneg = 1;
        break;
      
      default:
        break;
    }
  }
  
  
  TESTER_LED_Blink(&tester->leds.LED1);
  
}


/**
  * @brief    Обновление настроек тестера
  */
void TESTER_UpdateSettings(TESTER_TestHandleTypeDef *htest, MB_DataStructureTypeDef *mbdata)
{
  htest->SwTimings.ticks_before_disconnect.ticks = mbdata->HoldRegs.TimeBeforeDisconnect;
  htest->SwTimings.ticks_before_go_to_peak.ticks = mbdata->HoldRegs.TimeBeforePeak;
  htest->SwTimings.ticks_before_test.ticks = mbdata->HoldRegs.TimeBeforeTest;  
  htest->SwTimings.msticks_for_positive_dc = mbdata->HoldRegs.TimeForPositiveDC;
  
  htest->SwTimings.ticks_before_disconnect.msdelay = mbdata->Coils.msTimeBeforeDisconnect_enable;
  htest->SwTimings.ticks_before_go_to_peak.msdelay = mbdata->Coils.msTimeBeforePeak_enable;
  htest->SwTimings.ticks_before_test.msdelay = mbdata->Coils.msTimeBeforeTest_enable;  
  
  TESTER_ADC_UpdateSettings(htest->adc, mbdata);
}


/**
  * @brief    Инициализация тестера по дефолтным настрйокам в tester_config.h
  */
void TESTER_Set_Default_Settings(TESTER_ProjectTypeDef *tester)
{
  tester->mbdata->Coils.PositiveTest = 1;
  tester->mbdata->Coils.NegativeTest = 1;
  
  /* Настройка пинов для подключения отрицательного источника */
  tester->htest->DCNegSw.SW_Port    = SWITCH_PHASE_B_Port;
  tester->htest->DCNegSw.SwGND_Pin  = SWITCH_PHASE_B_HI_Pin;
  tester->htest->DCNegSw.SwVDD_Pin  = SWITCH_PHASE_B_LO_Pin;
  
  /* Настройка пинов для подключения положительного источника */
  tester->htest->DCPosSw.SW_Port    = SWITCH_PHASE_A_Port;
  tester->htest->DCPosSw.SwGND_Pin  = SWITCH_PHASE_A_HI_Pin;
  tester->htest->DCPosSw.SwVDD_Pin  = SWITCH_PHASE_A_LO_Pin;
  
  
  /* Настройка пинов для кнопки старта */
  tester->SwStart.Sw_Port           = SWITCH_START_Port;
  tester->SwStart.Sw_Pin            = SWITCH_START_Pin;
  tester->SwStart.Sw_FilterDelay    = SWITCH_ANTI_DREBEZG_DELAY;
  
  /* Настройка пинов для светодиода*/
  tester->leds.LED1.LED_Port        = LED1_Port;
  tester->leds.LED1.LED_Pin         = LED1_Pin;
  tester->leds.LED1.period          = LED_BLINK_AS_ON;
  
  
  /* Настройка таймингов по умолчанию для тестирования */
  tester->mbdata->HoldRegs.TimeForPositiveDC          = DEF_MS_TIME_FOR_POSITIVE;
  
  tester->mbdata->HoldRegs.TimeBeforeDisconnect       = DEF_TIME_BEFORE_DISCONNECT;
  tester->mbdata->Coils.msTimeBeforeDisconnect_enable = DEF_TIME_BEFORE_DISCONNECT_MS_DELAY;
  
  tester->mbdata->HoldRegs.TimeBeforePeak             = DEF_TIME_BEFORE_PEAK;
  tester->mbdata->Coils.msTimeBeforePeak_enable       = DEF_TIME_BEFORE_PEAK_MS_DELAY;
  
  tester->mbdata->HoldRegs.TimeBeforeTest             = DEF_TIME_BEFORE_TEST;
  tester->mbdata->Coils.msTimeBeforeTest_enable       = DEF_TIME_BEFORE_TEST_MS_DELAY;
  
  
  /* Настройка АЦП */
  tester->mbdata->HoldRegs.Adc_PulseWidth             = TESTER_ADC_PULES_EXPETCED_WIDTH;
  tester->mbdata->HoldRegs.Adc_CalibrValue            = ADC_VALUE_CALIBR;
  tester->mbdata->HoldRegs.Adc_ZeroValue              = ADC_VALUE_ZERO;
  tester->mbdata->HoldRegs.Adc_U_Calibr               = ADC_U_CALIBR;
}

/**
  * @brief    Задеркжка главного цикла
  */
void TESTER_main_delay(TESTER_ProjectTypeDef *tester)
{
  if(tester->delay_en)
    msDelay(tester->delay);
}